﻿'use strict';

/* Controllers */
var wdclient = angular.module('wdclient', ['ui.tree']);

wdclient.config(['$locationProvider', function ($locationProvider) {
    $locationProvider.html5Mode({
        enabled: true,
        rewriteLinks: false
    });
}]);

wdclient.filter('propertyFilter', function () {
    return function (input, search) {
        if (!input) return input;
        if (!search) return input;
        var expected = ('' + search).toLowerCase();
        var result = {};
        angular.forEach(input, function (value, key) {
            var actual = ('' + key).toLowerCase();
            if (actual.indexOf(expected) !== -1) {
                result[key] = value;
            }
        });
        return result;
    }
});

wdclient.filter('keyCount', function () {
    return function (input) {
        if (!angular.isObject(input)) {
            throw Error("Usage of non-objects with keylength filter!!")
        }
        return Object.keys(input).length;
    }
});

wdclient.filter('isKnownCertificate', function () {
    return function (input, certificates) {
        return certificates.some(function (c) { return c.thumbprint == input.thumbprint; });
    }
});

wdclient.directive('fileModel', ['$parse', function ($parse) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            var model = $parse(attrs.fileModel);
            var modelSetter = model.assign;

            element.bind('change', function () {
                scope.$apply(function () {
                    modelSetter(scope, element[0].files[0]);
                });
            });
        }
    };
}]);

wdclient.service('fileUpload', ['$http', function ($http) {
    this.uploadFileToUrl = function (file, uploadUrl) {
        var fd = new FormData();
        fd.append('files', file);
        return $http.post(uploadUrl, fd, {
            transformRequest: angular.identity,
            headers: { 'Content-Type': undefined }
        });
    }

    this.uploadFileToUrlWithPassword = function (file, uploadUrl, password) {
        var fd = new FormData();
        fd.append('password', password);
        fd.append('file', file);
        return $http.post(uploadUrl, fd, {
            transformRequest: angular.identity,
            headers: { 'Content-Type': undefined }
        });
    }

    this.uploadDeveloperDiskImage = function (file, signatureFile, uploadUrl) {
        var fd = new FormData();
        fd.append('DeveloperDiskImage.dmg', file);
        fd.append('DeveloperDiskImage.dmg.signature', signatureFile);
        return $http.post(uploadUrl, fd, {
            transformRequest: angular.identity,
            headers: { 'Content-Type': undefined }
        });
    }
}]);

wdclient.controller('MainController', ['$sce', '$scope', '$http', '$log', '$timeout', 'fileUpload', '$location', function ($sce ,$scope, $http, $log, $timeout, fileUpload, $location) {
    $scope.hideSettings = false;

    $scope.commandRequests =
    {
        "Click": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            }
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var firstElement = data.value[0].ELEMENT
                $http.post('/wd/hub/session/' + $scope.sessionId + '/element/' + firstElement + '/click', {})
            });
        },
        "SetProperty": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            }
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var firstElement = data.value[0].ELEMENT
                var setPropertyRequest = {
                    "propertyName": command.propertyName,
                    "propertyValue": command.propertyValue
                };

                $http.post('/wd/hub/session/' + $scope.sessionId + '/element/' + firstElement + '/property', setPropertyRequest);
            });
        },
        "GetProperty": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            }
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var firstElement = data.value[0].ELEMENT
                $http.get('/wd/hub/session/' + $scope.sessionId + '/element/' + firstElement + '/property/' + command.propertyName).success(function (data) {
                    window.alert("property: " + command.propertyName + " = " + data.value);
                });
            });
        },
        "TestElement": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            };
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var exist = data.value.length > 0;
                if (exist) {
                    window.alert("There is an element available with the given xPath.");
                }
                else {
                    window.alert("There is no element available with the given xPath.");
                }
            });
        },
        "TestProperty": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            };
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var firstElement = data.value[0].ELEMENT
                $http.get('/wd/hub/session/' + $scope.sessionId + '/element/' + firstElement + '/property/' + command.propertyName).success(function (data) {
                    window.alert("property: " + command.propertyName + " = " + data.value);
                });
            });
        },
        "GetElement": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            };
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var exist = data.value.length > 0;
                if (exist) {
                    window.alert("Element: " + data.value[0].ELEMENT);
                }
                else {
                    window.alert("There is no element available with the given xPath.");
                }
            });
        },
        "GetElements": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            };
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                window.alert("Element: " + data.value);
            });
        },
        "Back": (command) => {
            var gobackRequest = {};
            $http.post('/wd/hub/session/' + $scope.sessionId + '/back', gobackRequest);
        },
        "EnterText": (command) => {
            var findRequest = {
                "using": "xpath",
                "value": command.element.suggestedXPath,
                "ignoreVisibility": false
            }
            $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findRequest).success(function (data) {
                var firstElement = data.value[0].ELEMENT
                $http.post('/wd/hub/session/' + $scope.sessionId + '/element/' + firstElement + '/click', {}).success(function (data) {
                    $http.post('/wd/hub/session/' + $scope.sessionId + '/quamotion/clear', {}).success(function (data) {
                        var keysRequest = {
                            "value": [
                                command.text
                            ]
                        };
                        $http.post('/wd/hub/session/' + $scope.sessionId + '/keys', keysRequest).success(function (data) {
                            $http.post('/wd/hub/session/' + $scope.sessionId + '/quamotion/dismissKeyboard', {});
                        });
                    });
                });
            });
        },
        "SendKeys": (command) => {
            var keysRequest = {
                "value": [
                    command.text
                ]
            };
            $http.post('/wd/hub/session/' + $scope.sessionId + '/keys', keysRequest);
        },
        "DismissKeyboard": (command) => {
            $http.post('/wd/hub/session/' + $scope.sessionId + '/quamotion/dismissKeyboard', {});
        },
        "ClearText": (command) => {
            $http.post('/wd/hub/session/' + $scope.sessionId + '/quamotion/clear', {});
        }
    }

    $scope.commandTypes =
    {
        "model": null,
        "options": []
    };

    $scope.commands = [];

    $scope.editableCommand = null;

    $scope.executeCommand = function (command) {
        $scope.commandRequests[command.type.id](command);
    }

    $scope.executeCommandDefined = function (command) {
        return $scope.commandRequests[command.type.id];
    }

    $scope.parameterModel = function (parameter, command) {
        return function (value) {
            if (arguments.length) {
                if (parameter.id == "suggestedXPath") {
                    command.element = { "suggestedXPath": value };
                }
                command[parameter.id] = value;
            }
            else {
                return command[parameter.id];
            }
        }
    };

    $scope.getUrlParameter = function (name) {
        var uri = new URI(window.location.href);
        return uri.search(true).deviceId;
    };

    $scope.isCommandEditable = function (command) {
        return command == $scope.editableCommand;
    }

    $scope.hasParameters = function (command) {
        if (command && command.type && command.type.parameters) {
            return command.type.parameters.length > 0;
        }
        else {
            return false;
        }
    }

    $scope.showParameterType = function (type, parameter) {
        return parameter.type == type
    }

    $scope.addCommand = function () {
        if ($scope.commandTypes.model) {
            var command = { "type": $scope.commandTypes.model };
            $scope.commandTypes.model.parameters.forEach(function (parameter) {
                if (parameter["id"] == "appId" && $scope.selectedApp) {
                    $scope.parameterModel(parameter, command)($scope.selectedApp.AppId);
                }
                else if (parameter["id"] == "appVersion" && $scope.selectedApp) {
                    $scope.parameterModel(parameter, command)($scope.selectedApp.Version);
                }
                else if (parameter["id"] == "deviceId" && $scope.selectedDevice) {
                    $scope.parameterModel(parameter, command)($scope.selectedDevice.uniqueId);
                }
                else if (parameter["id"] == "suggestedXPath" && $scope.selectedWidget) {
                    $scope.parameterModel(parameter, command)($scope.selectedWidget.suggestedXPath);
                }
                else {
                    $scope.parameterModel(parameter, command)(null);
                }
            });

            $scope.commands.push(command);
            $scope.editableCommand = command;
        }
    }
    $scope.editCommand = function (command) {
        $scope.editableCommand = command;
    }

    $scope.removeCommand = function (command) {
        var index = $scope.commands.indexOf(command);
        $scope.commands.splice(index, 1);
    }

    $scope.hasCommands = function () {
        return $scope.commands.length > 0;
    }

    $scope.getTemplate = function (commandType, selectedLanguage) {
        var template = commandType.templates.filter(function (template) {
            return template.language == selectedLanguage;
        })[0];

        return template.expression;
    }

    // Recorder
    $scope.recordAvailable = function () {
        return !$scope.recordStarted && $scope.selectedPlatform != null && ($scope.selectedPlatform.name == 'Android' || $scope.selectedPlatform.name == 'iOS');
    }

    $scope.isRecordingPaused = false;
    $scope.recordStarted = false;
    $scope.startRecording = function () {
        $scope.recordStarted = true;
        var source = new EventSource('/wd/hub/session/' + $scope.sessionId + '/quamotion/record');

        source.onmessage = function (e) {
            if (!$scope.isRecordingPaused) {
                $scope.recordCommand(JSON.parse(e.data));
            }
        };
    }

    $scope.recordCommand = function (command) {
        var knownCommandType = $scope.setCommandType(command);
        if (knownCommandType) {
            if ($scope.commands.length > 0) {
                var lastRecordedCommand = $scope.commands[$scope.commands.length - 1];
                if (lastRecordedCommand.type.id == "EnterText" && command.actionName.Name == "EnterText" && command.element.elementId == lastRecordedCommand.element.elementId) {
                    lastRecordedCommand.text = command.text;
                    $scope.$apply();
                    return;
                }
            }

            if (command.element.suggestedXPath) {
                command.suggestedXPath = command.element.suggestedXPath;
            }

            $scope.commands.push(command);
            $scope.$apply();
        }
    }

    $scope.setCommandType = function (command) {
        if (command.actionName && command.actionName.Name) {
            var actionName = command.actionName.Name;
            var commandType = $scope.commandTypes.options.filter(function (commandType) {
                return commandType.id == actionName;
            })[0];
            if (commandType) {
                command.type = commandType;
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }

    $scope.getCommandTypes = function (command) {
        $http.get('/wd/hub/quamotion/commands').success(function (data) {
            $scope.commandTypes.options = data;
        });
    }

    $scope.getScriptFileName = function (selectedLanguage, name) {
        if (selectedLanguage == "Json") {
            return name + ".json";
        }
        else if (selectedLanguage == "PowerShell") {
            return name + ".ps1";
        }
        else {
            return name + ".txt";
        }
    }

    $scope.downloadScript = function (selectedLanguage, name) {
        var filename = $scope.getScriptFileName(selectedLanguage, name);
        var blob = new Blob([], { type: "text/json;charset=utf-8" });
        if (selectedLanguage == "Json") {
            var json = $scope.generateJson();
            blob = new Blob([json], { type: "text/json;charset=utf-8" });
        }
        else {
            var statements = $scope.generateCode(selectedLanguage);
            blob = new Blob([statements.join("\r\n")], { type: "text/json;charset=utf-8" });
        }

        saveAs(blob, filename);
    }

    $scope.generateCode = function (selectedLanguage) {
        var commands = [];
        $scope.commands.forEach(function (command) {
            var template = $scope.getTemplate(command.type, selectedLanguage);
            template.forEach(function (expression) {
                commands.push($scope.formatCommand(expression, command));
            });
        });
        return commands;
    }

    $scope.generateJson = function () {
        return "[" + $scope.commands.map(function (command) {
            var cmd = jQuery.extend(true, {}, command);
            delete cmd.smarty;
            delete cmd.type;
            delete cmd.widget;
            var actionName = { "Name": command.type.id };
            cmd.actionName = actionName;
            return angular.toJson(cmd);
        }).join(", \r\n") + "]";
    }

    // Determine the status
    $scope.getStatus = function () {
        // Get a list of all apps which are currently available
        $http.get('/wd/hub/status').then(function (response) {
            $scope.status = response.data.value;
            $scope.licenseErrors = $scope.status.licenseErrors;
            $scope.validLicense = $scope.licenseErrors.length == 0;
            $scope.developerDiskInstalled = $scope.status.dependencies.installedDeveloperDisks != null && $scope.status.dependencies.installedDeveloperDisks.length > 0;
        });
    }

    // Get a list of all developer certificates
    $scope.getCertificates = function () {
        // Get a list of all apps which are currently available
        $http.get('/wd/hub/quamotion/ios/certificates').then(function (response) {
            $scope.certificates = response.data;
        });
    }

    // Get a list of all provisioning profiles
    $scope.getProvisioningProfiles = function () {
        // Get a list of all apps which are currently available
        $http.get('/wd/hub/quamotion/ios/provisioningProfiles').then(function (response) {
            $scope.provisioningProfiles = response.data;
        });
    }


    // AutomationTypes

    $scope.$watch("selectedAutomationType", function (newValue, oldValue) {
        if (newValue && $scope.automationTypes.app) {
            $scope.showApplicationSettings = newValue.name == $scope.automationTypes.app.name;
        }
    });

    $scope.getAutomationTypes = function () {

        $scope.automationTypes = [];
        var app = { name: 'Application' };
        var web = { name: 'Web' };
        var device = { name: 'Device' };
        $scope.automationTypes =
        {
            "web": web,
            "device": device,
            "app": app
        };
    }

    $scope.setDefaultConfiguration = function () {
        if ($scope.selectedPlatform == null && $scope.devices != null && $scope.devices.length == 1) {
            $scope.selectedPlatform = $scope.platforms.filter(p => p.name == $scope.devices[0].configuration.OperatingSystem)[0];
        }
        if ($scope.devices == null || $scope.devices.length == 0) {
            $scope.selectedPlatform = null;
        }
    }

    $scope.$watch("selectedPlatform", function (newValue, oldValue) {
        if (($scope.selectedDevice == null && $scope.devices != null && $scope.devices.filter($scope.devicePlatformFilter).length == 1)) {
            $scope.selectedDevice = $scope.devices.filter($scope.devicePlatformFilter)[0];
        }
        else {
            $scope.selectedDevice = null;
        }

        if (($scope.selectedApp == null && $scope.apps != null && $scope.apps.filter($scope.appPlatformFilter).length == 1)) {
            $scope.selectedApp = $scope.apps.filter($scope.appPlatformFilter)[0]
        }
        else {
            $scope.selectedApp = null;
        }


        $scope.noDeviceWarningDisabled = newValue == null || ($scope.devices != null && $scope.devices.some($scope.devicePlatformFilter));
    });

    // Platform-related functionality.
    $scope.getPlatforms = function () {
        $scope.platforms = [];
        var android = { name: 'Android' };
        var ios = { name: 'iOS' };
        $scope.platforms.push(android);
        $scope.platforms.push(ios);
    }

    // Application-related functionality.
    $scope.getApps = function () {
        // Get a list of all apps which are currently available
        $http.get('/wd/hub/quamotion/app').then(function (response) {
            $scope.apps = response.data;
        });
    }

    $scope.importAppFromDevice = function ($event, deviceId, appId) {
        var l = Ladda.create($event.currentTarget);
        l.start();

        var importRequest = {
            deviceId: deviceId,
            appId: appId
        }

        $http.post('/wd/hub/quamotion/app/import/', importRequest).success(function (data) {
        }).error(function () {
            alert("An error occurred while importing the application.");
        })
            .finally(function () {
                l.stop();
            });
    }

    $scope.setSelectedApp = function (app) {
        $scope.selectedApp = app;
    }

    $scope.$watch("selectedApp", function (newValue, oldValue) {
        $scope.selectedAppProvisioningProfiles = null;
        if (newValue != null && newValue.SupportedConfigurations.OperatingSystem == 'iOS') {
            $http.get('/wd/hub/quamotion/app/' + newValue.SupportedConfigurations.OperatingSystem + '/' + newValue.AppId + '/' + newValue.Version + '/provisioningProfile').success(function (data) {
                $scope.selectedAppProvisioningProfiles = [data];
            });
        }
    });

    $scope.loadScenario = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#loadScenarioButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.myFile;
        if (file) {
            var aReader = new FileReader();
            aReader.readAsText(file, "UTF-8");
            aReader.onload = function (evt) {
                try {
                    var newCommands = angular.fromJson(aReader.result);
                    $scope.commands = [];
                    newCommands.forEach(function (command) {
                        var knownCommandType = $scope.setCommandType(command);
                        if (knownCommandType) {
                            $scope.commands.push(command);
                        }
                    });
                    $scope.$apply();
                    $('#loadScenarioModal').modal('hide');
                }
                catch (err) {
                    alert("An error occurred while loading the scenario. " + err.message);
                }
                l.stop();
            };
            aReader.onerror = function (evt) {
                if (l) {
                    l.stop();
                    alert("An error occurred while loading the scenario.");
                }
            };
        }
    }

    $scope.newApp = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#addAppButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.myFile;
        var uploadUrl = '/wd/hub/quamotion/app';
        fileUpload.uploadFileToUrl(file, uploadUrl)
            .success(function () {
                $('#newAppModal').modal('hide');
            })
            .error(function (error) {
                alert("An error occurred while uploading the file: " + error.Message);
            })
            .finally(function () {
                l.stop();
                $scope.getApps();
            });
    }

    $scope.resignDetails = null;
    $scope.getResignDetails = function () {
        // Submit a create session request to wd/hub/quamotion/resignDetails
        var appId = $location.search().appId;
        var deviceId = $location.search().deviceId;
        var appVersion = $location.search().appVersion;
        var resignDetailsRequest =
        {
            requiredCapabilities:
            {
                deviceId: deviceId,
                applicationType: "Native",
                appVersion: appVersion,
                appId: appId
            }
        };

        $http.post('/wd/hub/quamotion/resignDetails', resignDetailsRequest).then(function (response) {
            $scope.resignDetails = response.data;
        });
    }

    $scope.initializeDeveloperProfile = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#initializeDeveloperProfileButton');
        var l = Ladda.create(button);
        l.start();

        var initializeDeveloperProfileRequest =
        {
            accountName: $scope.initializeDeveloperProfile.accountName,
            accountPassword: $scope.initializeDeveloperProfile.accountPassword
        };

        $http.post('/wd/hub/quamotion/developercenter/initialize', initializeDeveloperProfileRequest)
            .success(function () {
                $('#initializeDeveloperProfileModal').modal('hide');
            })
            .error(function () {
                alert("An error occurred while initializing the developer profile.");
            })
            .finally(function () {
                l.stop();
                $scope.getCertificates();
                $scope.getProvisioningProfiles();
            });
    }

    $scope.importDeveloperDisk = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#importDeveloperDiskButton');
        var l = Ladda.create(button);
        l.start();

        var image = $scope.developerDiskImage;
        var signature = $scope.developerDiskSignature;
        var uploadUrl = '/wd/hub/quamotion/ios/developerDisk';
        fileUpload.uploadDeveloperDiskImage(image, signature, uploadUrl)
            .success(function () {
                $('#addDeveloperDiskModal').modal('hide');
            })
            .error(function () {
                alert("An error occurred while importing the developer disk.");
            })
            .finally(function () {
                l.stop();
                $scope.getStatus();
            });
    }

    $scope.importDeveloperProfile = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#importDeveloperProfileButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.developerProfileFile;
        var password = $scope.developerProfile.Password;
        var uploadUrl = '/wd/hub/quamotion/ios/developerProfile';
        fileUpload.uploadFileToUrlWithPassword(file, uploadUrl, password)
            .success(function () {
                $('#importDeveloperProfileModal').modal('hide');
            })
            .error(function () {
                alert("An error occurred while importing the developer profile.");
            })
            .finally(function () {
                l.stop();
                $scope.getCertificates();
                $scope.getProvisioningProfiles();
            });
    }

    $scope.importCertificate = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#importCertificateButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.certificateFile;
        var password = $scope.certificate.Password;
        var uploadUrl = '/wd/hub/quamotion/ios/certificate';
        fileUpload.uploadFileToUrlWithPassword(file, uploadUrl, password)
            .success(function () {
                $('#addCertificateModal').modal('hide');
            })
            .error(function () {
                alert("An error occurred while importing the developer identity.");
            })
            .finally(function () {
                l.stop();
                $scope.getCertificates();
            });
    }

    $scope.importProvisioningProfile = function () {
        // Create the ladda (spinning wheel) function
        var button = document.querySelector('#importProvisioningProfileButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.provisioningProfileFile;
        var uploadUrl = '/wd/hub/quamotion/ios/provisioningProfile';
        fileUpload.uploadFileToUrl(file, uploadUrl)
            .success(function () {
                $('#addProvisioningProfileModal').modal('hide');
            })
            .error(function () {
                alert("An error occurred while importing the provisioning profile.");
            })
            .finally(function () {
                l.stop();
                $scope.getProvisioningProfiles();
            });
    }

    $scope.deleteCertificate = function (certificate) {
        $http.delete('/wd/hub/quamotion/ios/certificate/' + certificate.thumbprint)
            .then(function (response) { $scope.getCertificates() });
    }

    $scope.deleteProvisioningProfile = function (provisioningProfile) {
        $http.delete('/wd/hub/quamotion/ios/provisioningProfile/' + provisioningProfile.uuid)
            .then(function (response) { $scope.getProvisioningProfiles() });
    }

    $scope.removeApplication = function (operatingSystem, appId, versionId) {
        $http.delete('/wd/hub/quamotion/app/' + operatingSystem + '/' + appId + '/' + versionId)
            .then(function (response) {
                $scope.getApps();
            });
    }

    $scope.viewPort = null;
    $scope.getViewPort = function () {
        if ($scope.sessionId == null) {
            return;
        }

        $http.get('/wd/hub/session/' + $scope.sessionId + '/window/size').then(function (response) {
            $scope.viewPort = response.data.value;
        });
    }

    $scope.removeAppWithConfirmation = function () {
        var button = $('#removeAppButton')[0];
        var buttonText = button.innerText;
        var confirmationText = "Are you sure to remove this app";
        var standardText = "Remove";
        if (buttonText == confirmationText) {
            button.innerText = standardText;
            button.classList = [];
            button.classList.add("btn");
            button.classList.add("btn-info");
            $scope.removeApplication($scope.selectedApp.SupportedConfigurations.OperatingSystem, $scope.selectedApp.AppId, $scope.selectedApp.Version);
            $('#viewAppModal').modal('hide');
        }
        else {
            button.innerText = confirmationText;
            button.classList = [];
            button.classList.add("btn");
            button.classList.add("btn-warning");
        }
    };

    $scope.requestLicense = function () {
        var button = document.querySelector('#requestLicenseButton');
        var l = Ladda.create(button);
        l.start();

        var request =
        {
            Email: $scope.licenseRequest.email,
            FirstName: $scope.licenseRequest.firstName,
            LastName: $scope.licenseRequest.lastName,
            Company: $scope.licenseRequest.company,
            JobTitle: $scope.licenseRequest.job,
            OfficePhone: $scope.licenseRequest.phone,
            OrderId: $scope.licenseRequest.orderId
        };

        delete $scope.licenseRequestError;
        $http.post('/wd/hub/quamotion/license/request', request)
            .success(function (data) {
                $scope.getStatus();
            })
            .error(function (data) {

                var error = "We're sorry, we could not generate your license. Please contact us at info@quamotion.mobi to place a new order.";
                if (data.value && data.value.message) {
                    error = "We're sorry, we could not generate your license.<BR>" + data.value.message;
                }

                $scope.licenseRequestError = $sce.trustAsHtml(error); 
            })
            .finally(function () {
                l.stop();
            });
    };

    $scope.updateLicense = function () {
        var button = document.querySelector('#updateLicenseButton');
        var l = Ladda.create(button);
        l.start();

        var file = $scope.myFile;
        var uploadUrl = '/wd/hub/quamotion/license';
        fileUpload.uploadFileToUrl(file, uploadUrl)
            .success(function () {
                $('#updateLicenseModal').modal('hide');
                $scope.getStatus();

            })
            .error(function () {
                alert("An error occurred while uploading the license.");
            })
            .finally(function () {
                l.stop();
            });

        l.stop();
    }

    $scope.updateWidgetHighlight = function () {
        if ($scope.selectedWidget == null || $scope.selectedDevice == null) {
            return;
        }

        var selectedDevice = $scope.selectedDevice;
        var deviceWidth = $scope.viewPort.width;
        var deviceHeight = $scope.viewPort.height;

        var containerElement = document.getElementById("feedContainer");
        var containerHeight = containerElement.clientHeight;
        var containerWidth = containerElement.clientWidth;

        var heightFactor = containerHeight / deviceHeight;
        var widthFactor = containerWidth / deviceWidth;

        var highlightElement = document.getElementById("highlightedElement");

        var widgetRectangle = $scope.selectedWidget.rectangle;

        highlightElement.style.left = widgetRectangle.x * widthFactor + "px";
        highlightElement.style.top = widgetRectangle.y * heightFactor + "px";
        highlightElement.style.height = widgetRectangle.height * heightFactor + "px";
        highlightElement.style.width = widgetRectangle.width * widthFactor + "px";
    }

    $scope.widgetSelected = function () {
        return $scope.selectedWidget != null;
    }

    $scope.resolveWidget = function (command) {
        var xpath = command.element.suggestedXPath;
        if (xpath && xpath != "") {
            $scope.findWidgetByXPath(xpath, function (widget) {
                command.widget = widget;
                $scope.resolveProperties(widget);
            });
        }
    }

    $scope.selectWidgetByXPath = function (xpath) {
        $scope.findWidgetByXPath(xpath, function (widget) { $scope.selectWidget(widget) });
    }

    $scope.findWidgetByXPath = function (xpath, assign) {
        var findElementsRequest =
        {
            using: "xpath",
            value: xpath,
            ignoreVisibility: false
        };
        // find element by xpath
        $http.post('/wd/hub/session/' + $scope.sessionId + '/elements', findElementsRequest).success(function (data) {
            if (data.value[0]) {
                // check if we can find the element id in the current source tree
                var widget = $scope.findWidgetByElementId($scope.widgets, data.value[0].ELEMENT);
                if (widget != null) {
                    // widget found in source tree with same element id
                    widget.xpath = xpath;
                    assign(widget);
                }
                else {
                    // no widget found in source tree which corresponds to the element id (in FDA we do not have element id info)
                    // request element info (rectangle)
                    $http.get('/wd/hub/session/' + $scope.sessionId + '/element/' + data.value[0].ELEMENT).success(function (elementData) {
                        widget = elementData.value;
                        if (widget != null) {
                            widget.xpath = xpath;
                            assign(widget);
                        }
                        else {
                            assign(null);
                        }
                    });
                }
            }
            else {
                // no element is found which corresponds to the given xpath
                assign(null);
            }
        });
    }

    $scope.findWidgetByElementId = function (widgets, elementId) {
        var result = null;

        widgets.forEach(function (widget) {
            if (widget.elementId == elementId) {
                result = widget;
            }
            else {
                if (widget.children && result == null) {
                    result = $scope.findWidgetByElementId(widget.children, elementId);
                }
            }
        });

        return result;
    }

    $scope.selectWidget = function (widget) {
        $scope.selectedWidget = widget;
        $scope.updateWidgetHighlight();

        if (!widget.xpath || widget.xpath == "") {
            widget.xpath = widget.suggestedXPath;
        }
    }

    $scope.showWidget = function () {
        var canvasContext = $scope.initFeedCanvas();

        var elementRectangle = $scope.selectedWidget.rectangle;


        canvasContext.rect(elementRectangle.x, elementRectangle.y, elementRectangle.width, elementRectangle.height);
        canvasContext.strokeStyle = "red";
        canvasContext.lineWidth = "2";
        canvasContext.stroke();
    }

    // Device-related functionality
    $scope.getDevices = function () {
        $http.get('/wd/hub/quamotion/device').then(function (response) {
            $scope.devices = response.data;
            $scope.setDefaultConfiguration();
        });
    }

    $scope.setSelectedDevice = function (device) {
        $scope.selectedDevice = device;
    }

    $scope.$watch("selectedDevice", function (newValue, oldValue) {
        $scope.getInstalledApps(newValue.uniqueId);
        $scope.noDeviceSelected = newValue == null;
    });

    $scope.installedApps = null;
    // Application-related functionality.
    $scope.getInstalledApps = function (deviceId) {
        if (deviceId == null) {
            $scope.installedApps = null;
        }
        else {
            // Get a list of all apps which are currently available
            $http.get('/wd/hub/quamotion/device/' + deviceId + '/app').then(function (response) {
                $scope.installedApps = response.data;
            });
        }
    }

    $scope.reuseExistingSession = true;

    $scope.appPlatformFilter = function (app, index, array) {
        return $scope.selectedPlatform == null || app.SupportedConfigurations.OperatingSystem == $scope.selectedPlatform.name;
    }

    $scope.devicePlatformFilter = function (device, index, array) {
        return $scope.selectedPlatform == null || device.configuration.OperatingSystem == $scope.selectedPlatform.name;
    }

    // Session-related functionality
    $scope.getSessions = function () {
        $http.get('/wd/hub/sessions').then(function (response) {
            $scope.sessions = response.data.value;
        });
    }

    $scope.removeSession = function (sessionId) {
        $http.delete('/wd/hub/session/' + sessionId).then(function (response) {
        });
    }

    $scope.sortType = 'name'; // set the default sort type
    $scope.sortReverse = false;  // set the default sort order
    $scope.searchKey = '';     // set the default search/filter term

    $scope.startOrRecycleSession = function () {
        var button = document.querySelector('#startSessionButton');
        $scope.startSession();
    }

    // Starts a new session, using the device and application the user selected.
    $scope.startSession = function () {
        $scope.createSessionFailedErrorDisabled = true;
        $scope.recordStarted = false;
        $scope.isRecordingPaused = false;
        $scope.commands = [];
        $scope.selectedWidget = null;

        var createSessionRequest = null;

        if ($scope.selectedApp != null && $scope.selectedAutomationType.name == $scope.automationTypes.app.name) {
            createSessionRequest =
            {
                requiredCapabilities:
                {
                    deviceId: $scope.selectedDevice.uniqueId,
                    applicationType: "Native",
                    appId: $scope.selectedApp.AppId,
                    appVersion: $scope.selectedApp.Version,
                    reuseExistingSession: $scope.reuseExistingSession
                }
            };
        }
        else if ($scope.selectedAutomationType.name == $scope.automationTypes.device.name) {
            createSessionRequest =
            {
                requiredCapabilities:
                {
                    deviceId: $scope.selectedDevice.uniqueId,
                    applicationType: "Device",
                    reuseExistingSession: $scope.reuseExistingSession
                }
            };
        }
        else if ($scope.selectedAutomationType.name == $scope.automationTypes.web.name) {
            createSessionRequest =
            {
                requiredCapabilities:
                {
                    deviceId: $scope.selectedDevice.uniqueId,
                    applicationType: "Web",
                    reuseExistingSession: $scope.reuseExistingSession
                }
            };
        }

        $scope.isDeploying = true;
        $scope.noSessionAvailable = true;
        $scope.sessionStatus = "";
        $scope.sessionStatusMessage = "";
        $('#startingSessionModal').modal('show');

        $http.post('/wd/hub/session', createSessionRequest).success(function (data) {
            $scope.sessionId = data.sessionId;

            (function tick() {
                $http.get('/wd/hub/session/' + $scope.sessionId + "/quamotion/status").success(function (data) {
                    $log.info("Received session data: " + data);
                    $scope.sessionStatus = data.value.Status;
                    $scope.sessionStatusMessage = data.value.StatusMessage;

                    if ($scope.sessionStatus == "Deploying") {
                        $timeout(tick, 1000);
                    } else {
                        $scope.isDeploying = false;
                        $('#startingSessionModal').modal('hide');
                        $scope.createSessionFailedErrorDisabled = $scope.sessionStatus != 'DeployFailed';
                        $scope.noSessionAvailable = $scope.sessionStatus == 'DeployFailed';
                        $scope.getViewPort();
                    }
                });
            })();
        }).error(function () {
            $scope.isDeploying = false;
            $('#startingSessionModal').modal('hide');
            $scope.createSessionFailedErrorDisabled = false;
            $scope.noSessionAvailable = true;
        });
    };

    // Performs a spy on the current page.
    $scope.doSpy = function () {
        $scope.spyPoint = null;
        $scope.selectedWidget = null;
        $scope.updateWidgets();
    }

    // updates the widgets (loads the treeview of all HTML elements).
    $scope.updateWidgets = function () {
        if ($scope.sessionId != null) {
            $http.get('/wd/hub/session/' + $scope.sessionId + '/source').then(function (response) {
                $scope.widgets = response.data.value;
                $scope.buildSpyTree();
                $scope.UpdateSelectedWidget($scope.widgets);
            });
        }
    };

    $scope.UpdateSelectedWidget = function (widgets) {
        var point = $scope.spyPoint;
        var index = 0;
        while (index < widgets.length && !$scope.spyMatch(widgets[index], point)) {
            index++;
        }
        $scope.selectWidget($scope.FindChild(widgets[index]));
    }

    $scope.FindChild = function (widget) {
        var point = $scope.spyPoint;
        var index = widget.children.length - 1;
        while (index > 0 && !$scope.spyMatch(widget.children[index], point)) {
            index--;
        }

        if (index >= 0) {
            var last = widget.children[index];
            return $scope.FindChild(last);
        }
        else {
            return widget;
        }
    }

    // convert to data model used by boostrap-treeview (uses nodes in stead of childeren)
    $scope.createSpyNode = function (widget) {
        if (!widget) { return; }

        // set text
        var node = {
            text: $scope.getSpyInfo(widget),
            widget: widget
        }

        // expand if object is visible
        if (widget.visible) {
            node.state =
            {
                expanded: true
            };
        }

        // convert also all children
        if (widget.children && widget.children.length > 0) {
            node.nodes = [];
            $.each(widget.children, function (id, widget) {
                if ($scope.spyMatch(widget, $scope.spyPoint)) {
                    var childNode = $scope.createSpyNode(widget);
                    node.nodes.push(childNode);
                }
            });

        }

        return node;
    }

    // build the spy tree and make and initialize the bootstrap-treeview
    $scope.buildSpyTree = function () {
        // convert all widgets to data model used by boostrap-treeview
        var data = [];
        $.each($scope.widgets, function (id, widget) {
            var node = $scope.createSpyNode(widget);
            if (node != null) {
                data.push(node);
            }
        });

        // set treeview
        var options = {
            bootstrap2: true,
            data: data
        };

        $('#spyTree').treeview(options);
        $('#spyTree').on('nodeSelected', function (event, data) {
            $scope.$apply(function () {
                $scope.selectWidget(data.widget);
            });
        });
    }

    // gets the treenode info
    $scope.getSpyInfo = function (widget) {
        var info = "<code>" + widget.suggestedXPath + "</code>"
        if (widget.id) {
            info = "<strong>" + widget.id + "</strong>" + info;
        }

        return info;
    }

    $scope.$watch("selectedWidget", function (newValue, oldValue) {
        $scope.resolveProperties(newValue);
    });

    $scope.resolveProperties = function (widget) {
        if ($scope.sessionId && widget && widget.elementId) {
            $http.get('/wd/hub/session/' + $scope.sessionId + '/element/' + widget.elementId + '/property/').success(function (data) {
                widget.properties = data.value;
            });
        }
    }

    $scope.$watch("propertySearch", function (newValue, oldValue) {
        if (!newValue) {
            $scope.propertySearchValue = "";
        }
        if ($scope.sessionId && $scope.selectedWidget && $scope.selectedWidget.elementId && newValue) {
            $http.get('/wd/hub/session/' + $scope.sessionId + '/element/' + $scope.selectedWidget.elementId + '/property/' + newValue).success(function (data) {
                if ($scope.propertySearch == newValue) {
                    $scope.propertySearchValue = data.value;
                }
            });
        }
    });

    // Identifies a widget after the user has selected the widget.
    $scope.findWidget = function (e) {
        var resolution = $scope.viewPort;
        var elementWitdh = e.target.offsetWidth;
        var elementHeight = e.target.offsetHeight;

        var x = e.offsetX * resolution.width / elementWitdh;
        var y = e.offsetY * resolution.height / elementHeight;

        $scope.spyPoint =
        {
            X: x,
            Y: y
        };

        $scope.updateWidgets();
    };

    // Finds a widget after the user has clicked on a widget in the device screen.

    $scope.spyMatch = function (widget, point) {
        var match = false;

        if ($scope.ContainsPoint(widget, point)) {
            return true;
        }
        else {
            var index = 0;
            while (index < widget.children.length && !match) {
                match = match || $scope.spyMatch(widget.children[index], point);
                index++;
            }
        }

        return match;
    };

    $scope.ContainsPoint = function (widget, point) {
        return !point || (point.X > widget.rectangle.x &&
            point.X < (widget.rectangle.x + widget.rectangle.width) &&
            point.Y > widget.rectangle.y &&
            point.Y < (widget.rectangle.y + widget.rectangle.height) &&
            widget.visible == true);
    }

    $scope.formatCommand = function (statement, data) {
        if (statement == "") {
            return angular.toJson(data);
        }
        else {
            var compiledTemplate = new jSmart(statement);
            return compiledTemplate.fetch(data);
        }
    }

    $scope.getStatus();
    $scope.buildSpyTree();
    $scope.getPlatforms();
    $scope.getAutomationTypes();
    $scope.getApps();
    $scope.getDevices();
    $scope.getCommandTypes();
    $scope.getSessions();
    $scope.getCertificates();
    $scope.getProvisioningProfiles();

    $scope.selectedPlatform = null;
    $scope.createSessionFailedErrorDisabled = true;
    $scope.noSessionAvailable = true;

    // Results-related functionality

    $scope.uploadResult = function () {
        var file = $scope.uploadResultsFile;
        var uploadUrl = '/wd/hub/quamotion/testlog';
        fileUpload.uploadFileToUrl(file, uploadUrl);

        l.stop();
    }

    var NotificationTypeEnum = {
        Devices: "Devices",
        Applications: "Applications",
        Sessions: "Sessions"
    };

    // Initialize the websocket.
    $scope.initializeNotifications = function () {
        var loc = window.location;
        var wsProtocol = null;

        if (loc.protocol === "https:") {
            wsProtocol = "wss://";
        } else {
            wsProtocol = "ws://";
        }

        var notificationURL = wsProtocol + window.location.host + "/wd/hub/quamotion/notifications";
        var notificationSocket = new WebSocket(notificationURL);
        notificationSocket.onmessage = function (e) { $scope.updateScope(e.data); };
        notificationSocket.onerror = function (e) { $scope.updateScope(e.data); };
    }

    $scope.updateScope = function (jsonNotification) {
        var notification = JSON.parse(jsonNotification);
        switch (notification.notificationType) {
            case NotificationTypeEnum.Devices:
                $scope.getDevices();
                break;
            case NotificationTypeEnum.Applications:
                $scope.getApps();
                break;
            case NotificationTypeEnum.Sessions:
                $scope.getSessions();
                break;
        }
    }

    $scope.initializeNotifications();
}]);